package dcnet

import (
	"bufio"
	"bytes"
	"fmt"
	"log"
	"net/http"
	"net/http/httputil"
)

func SetBit(n uint, b bool, bs []byte) {
	if b {
		bs[n/8] |= 1 << (n % 8)
	} else {
		bs[n/8] &= ^(1 << (n % 8))
	}
}

func Xor(r []byte, w []byte) {
	for i, b := range r {
		w[i] ^= b
	}
}

func Xors(bss [][]byte) []byte {
	n := len(bss[0])
	x := make([]byte, n)
	for _, bs := range bss {
		for i, b := range bs {
			x[i] ^= b
		}
	}
	return x
}

func CString(s []byte) string {
	n := bytes.Index(s, []byte{0})
	if n != -1 {
		s = s[:n]
	}
	return string(s)
}

func FetchURL(url string) []byte {
	fmt.Println("Fetching URL:", url)
	resp, err := http.Get(url)
	if err != nil {
		// TODO ignored for now
	}

	defer resp.Body.Close()
	response := make([]byte, RspLen)
	resp.Body.Read(response) // TODO check error
	if err != nil {
		// TODO ignored for now
	}
	return response
}

// TODO error handling really needs to be improved for the Fetch* functions!
func FetchHttpReq(request []byte) []byte {
	reqBuf := bufio.NewReader(bytes.NewReader(request))

	req, err := http.ReadRequest(reqBuf)
	if err != nil {
		log.Fatal("Couldn't generate request: ", err)
	}

	response := make([]byte, RspLen)
	if req.Method == "CONNECT" {
		// currently ignoring connect response (associated with https)
		return response
	}

	fmt.Println("Fetching URL:", req.RequestURI)
	req, _ = http.NewRequest(req.Method, req.RequestURI, req.Body)

	client := &http.Client{}
	resp, err := client.Do(req)
	req.Body.Close()
	if err != nil {
		return response
	}

	respArr, err := httputil.DumpResponse(resp, true)
	if err != nil {
		return response
	}
	defer resp.Body.Close()

	copy(response, respArr)

	return response
}

func AllZero(xs []byte) bool {
	for _, x := range xs {
		if x != 0 {
			return false
		}
	}
	return true
}

func FetchReqs(reqs [][]byte) [][]byte {
	responses := make([][]byte, len(reqs))

	for i, req := range reqs {
		if AllZero(req) {
			responses[i] = make([]byte, RspLen)
		} else if bytes.HasPrefix(req, []byte("http")) {
			// TODO this is an ugly hack!
			// we need it to support the cmd client mode
			responses[i] = FetchURL(CString(req))
		} else {
			fmt.Println(CString(req))
			responses[i] = FetchHttpReq(req)
		}
	}

	return responses
}

// enable sorting of []uint32 slices (based on code from the sort package)
type Uint32Slice []uint32

func (p Uint32Slice) Len() int           { return len(p) }
func (p Uint32Slice) Less(i, j int) bool { return p[i] < p[j] }
func (p Uint32Slice) Swap(i, j int)      { p[i], p[j] = p[j], p[i] }
